//-----------------------------------------------------------------
// Shaper View Object
// C++ Source - ShaperView.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Inclusions
//-----------------------------------------------------------------
#include "StdAfx.h"
//-----------------------------------------------------------------
#include "ShaperDoc.h"
#include "ShaperView.h"
#include "Resource.h"

//-----------------------------------------------------------------
// MFC Debugging Support
//-----------------------------------------------------------------
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------
// Message Map & Runtime Support
//-----------------------------------------------------------------
BEGIN_MESSAGE_MAP(CShaperView, CScrollView)
  ON_WM_LBUTTONDOWN()
  ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
  ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
  ON_COMMAND(ID_DRAW_LINE, OnDrawLine)
  ON_COMMAND(ID_DRAW_RECTANGLE, OnDrawRectangle)
  ON_COMMAND(ID_DRAW_TRIANGLE, OnDrawTriangle)
  ON_COMMAND(ID_DRAW_ELLIPSE, OnDrawEllipse)
  ON_COMMAND(ID_DRAW_CHANGECOLOR, OnDrawChangeColor)
  ON_UPDATE_COMMAND_UI(ID_DRAW_LINE, OnUpdateDrawLine)
  ON_UPDATE_COMMAND_UI(ID_DRAW_RECTANGLE, OnUpdateDrawRectangle)
  ON_UPDATE_COMMAND_UI(ID_DRAW_TRIANGLE, OnUpdateDrawTriangle)
  ON_UPDATE_COMMAND_UI(ID_DRAW_ELLIPSE, OnUpdateDrawEllipse)
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CShaperView, CScrollView)

//-----------------------------------------------------------------
// Public Constructor(s)/Destructor
//-----------------------------------------------------------------
CShaperView::CShaperView() : m_nDrawMode(CShape::LINE),
  m_crColor(RGB(0, 0, 0)) {
}

CShaperView::~CShaperView() {
}

//-----------------------------------------------------------------
// Public Member Functions
//-----------------------------------------------------------------
#ifdef _DEBUG
CShaperDoc* CShaperView::GetDocument() {
  ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CShaperDoc)));
  return (CShaperDoc*)m_pDocument;
}
#endif //_DEBUG

void CShaperView::OnDraw(CDC* pDC) {
  // Get a pointer to the document
  CShaperDoc* pDoc = GetDocument();
  ASSERT_VALID(pDoc);

  // Get the clipping rect for the DC
  CRect rcClip, rcShape;
  pDC->GetClipBox(&rcClip);
  rcClip.top = -rcClip.top;
  rcClip.bottom = -rcClip.bottom;
  
  // Draw the view (paint the shapes)
  POSITION pos = pDoc->m_shapeList.GetHeadPosition();
  while (pos != NULL) {
    // Get the next shape
    CShape* pShape = pDoc->m_shapeList.GetNext(pos);

    // Only draw if the shape rect intersects the clipping rect
    rcShape = pShape->GetPosition();
    rcShape.top = -rcShape.top;
    rcShape.bottom = -rcShape.bottom;
    if (rcShape.IntersectRect(&rcShape, &rcClip))
      pShape->Draw(pDC);
  }
}

void CShaperView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) {
  // Make sure the hint is valid
  if (pHint != NULL) {
    if (pHint->IsKindOf(RUNTIME_CLASS(CShape))) {
      // Update the scroll sizes
      CShaperDoc* pDoc = GetDocument();
      ASSERT_VALID(pDoc);
      SetScrollSizes(MM_LOENGLISH, pDoc->GetDocSize());

      // Invalidate only the rectangular position of the new shape
      CShape* pShape = DYNAMIC_DOWNCAST(CShape, pHint);
      CClientDC dc(this);
      OnPrepareDC(&dc);
      CRect rc = pShape->GetPosition();
      dc.LPtoDP(&rc);
      rc.InflateRect(1, 1);
      InvalidateRect(&rc);
      return;
    }
  }

  // Invalidate the entire view
  Invalidate();
}

void CShaperView::OnInitialUpdate() {
  // Set the scroll sizes
  CShaperDoc* pDoc = GetDocument();
  ASSERT_VALID(pDoc);
  SetScrollSizes(MM_LOENGLISH, pDoc->GetDocSize());
}

void CShaperView::OnPrint(CDC* pDC, CPrintInfo* pInfo) {
  // Get a pointer to the document
  CShaperDoc* pDoc = GetDocument();
  ASSERT_VALID(pDoc);

  // Print the page header and adjust the DC window origin
  CString sDocTitle = pDoc->GetTitle();
  PrintPageHeader(pDC, pInfo, sDocTitle);
  pDC->SetWindowOrg(pInfo->m_rectDraw.left, -pInfo->m_rectDraw.top);

  // Print the document data
  OnDraw(pDC);
}

//-----------------------------------------------------------------
// Protected Member Functions
//-----------------------------------------------------------------
BOOL CShaperView::OnPreparePrinting(CPrintInfo* pInfo) {
  return DoPreparePrinting(pInfo);
}

void CShaperView::PrintPageHeader(CDC* pDC, CPrintInfo* pInfo,
  CString& sHeader) {
  // Draw the header text aligned left
  pDC->SetTextAlign(TA_LEFT);
  pDC->TextOut(0, -25, sHeader);
  
  // Draw a line across the page just below the header text
  TEXTMETRIC tm;
  pDC->GetTextMetrics(&tm);
  int y = -35 - tm.tmHeight;
  pDC->MoveTo(0, y);
  pDC->LineTo(pInfo->m_rectDraw.right, y);

  // Adjust the drawing rect to not include the header
  y -= 25;
  pInfo->m_rectDraw.top += y;
}

//-----------------------------------------------------------------
// Message Handlers
//-----------------------------------------------------------------
void CShaperView::OnLButtonDown(UINT nFlags, CPoint point) {
  // Get a pointer to the document
  CShaperDoc* pDoc = GetDocument();
  ASSERT_VALID(pDoc);

  // Start the "rubber band" rectangle tracker
  CRectTracker tracker;
  tracker.TrackRubberBand(this, point);

  // Convert the tracker rect from device to logical coordinates
  CClientDC dc(this);
  OnPrepareDC(&dc);
  dc.DPtoLP(&tracker.m_rect);

  // Add new shape based on the drawing mode
  switch (m_nDrawMode) {
  case CShape::LINE:
    // Add a line to the shape list
    pDoc->AddShape(new CShape(CShape::LINE, tracker.m_rect,
      m_crColor));
    break;
  
  case CShape::RECTANGLE:
    // Add a rectangle to the shape list
    pDoc->AddShape(new CShape(CShape::RECTANGLE, tracker.m_rect,
      m_crColor));
    break;

  case CShape::TRIANGLE:
    // Add a triangle to the shape list
    pDoc->AddShape(new CShape(CShape::TRIANGLE, tracker.m_rect,
      m_crColor));
    break;

  case CShape::ELLIPSE:
    // Add an ellipse to the shape list
    pDoc->AddShape(new CShape(CShape::ELLIPSE, tracker.m_rect,
      m_crColor));
    break;
  }
}

void CShaperView::OnDrawLine() {
  // Set the drawing mode to Line
  m_nDrawMode = CShape::LINE;
}

void CShaperView::OnDrawRectangle() {
  // Set the drawing mode to Rectangle
  m_nDrawMode = CShape::RECTANGLE;
}

void CShaperView::OnDrawTriangle() {
  // Set the drawing mode to Triangle
  m_nDrawMode = CShape::TRIANGLE;
}

void CShaperView::OnDrawEllipse() {
  // Set the drawing mode to Ellipse
  m_nDrawMode = CShape::ELLIPSE;
}

void CShaperView::OnDrawChangeColor() {
  // Select a pen color using the Color dialog box
  CColorDialog dlg(m_crColor);
  if (dlg.DoModal() == IDOK) {
    // Change the current color
    m_crColor = dlg.GetColor();
  }
}

void CShaperView::OnUpdateDrawLine(CCmdUI* pCmdUI) {
  // Update the Draw|Line menu item
  pCmdUI->SetRadio(m_nDrawMode == CShape::LINE);
}

void CShaperView::OnUpdateDrawRectangle(CCmdUI* pCmdUI) {
  // Update the Draw|Rectangle menu item
  pCmdUI->SetRadio(m_nDrawMode == CShape::RECTANGLE);
}

void CShaperView::OnUpdateDrawTriangle(CCmdUI* pCmdUI) {
  // Update the Draw|Triangle menu item
  pCmdUI->SetRadio(m_nDrawMode == CShape::TRIANGLE);
}

void CShaperView::OnUpdateDrawEllipse(CCmdUI* pCmdUI) {
  // Update the Draw|Ellipse menu item
  pCmdUI->SetRadio(m_nDrawMode == CShape::ELLIPSE);
}

//-----------------------------------------------------------------
// Diagnostic Functions
//-----------------------------------------------------------------
#ifdef _DEBUG
void CShaperView::AssertValid() const {
  CScrollView::AssertValid();
}

void CShaperView::Dump(CDumpContext& dc) const {
  CScrollView::Dump(dc);
}
#endif //_DEBUG
