//-----------------------------------------------------------------
// Main Frame Window Object
// C++ Source - MainFrame.cpp
//-----------------------------------------------------------------

//-----------------------------------------------------------------
// Inclusions
//-----------------------------------------------------------------
#include "StdAfx.h"
//-----------------------------------------------------------------
#include "MainFrame.h"
#include "Wave.h"
#include "Resource.h"
#include <StdLib.h>

//-----------------------------------------------------------------
// MFC Debugging Support
//-----------------------------------------------------------------
#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif


//-----------------------------------------------------------------
// CMainFrame Message Map & Runtime Support
//-----------------------------------------------------------------
BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
  ON_WM_CREATE()
  ON_WM_DESTROY()
  ON_WM_PAINT()
  ON_WM_TIMER()
END_MESSAGE_MAP()

IMPLEMENT_DYNCREATE(CMainFrame, CFrameWnd)

//-----------------------------------------------------------------
// CMainFrame Public Constructor(s)/Destructor
//-----------------------------------------------------------------
CMainFrame::CMainFrame()
  : m_pDirectSound(NULL), m_uiTimerID(0), m_lHeliPan(0),
    m_lHeliPanInc(0) {
  // Initialize member variables
  for (int i = 0; i < NUMSOUNDS; i++)
    m_pDSBuffer[i] = NULL;
}

CMainFrame::~CMainFrame() {
  CleanupDirectSound();
}

//-----------------------------------------------------------------
// Public Methods
//-----------------------------------------------------------------
BOOL CMainFrame::Create(const CString& sTitle) {
  CString sClassName;

  sClassName = AfxRegisterWndClass(CS_HREDRAW | CS_VREDRAW,
    LoadCursor(NULL, IDC_ARROW),
    (HBRUSH)(COLOR_WINDOW + 1),
    LoadIcon(AfxGetInstanceHandle(),
    MAKEINTRESOURCE(IDR_WAR)));

  return CFrameWnd::Create(sClassName, sTitle);
}

//-----------------------------------------------------------------
// Protected Methods
//-----------------------------------------------------------------
BOOL CMainFrame::InitDirectSound() {
  // Create the DS object
  if (::DirectSoundCreate(NULL, &m_pDirectSound, NULL) != DS_OK) {
    MessageBox("Could not create DirectSound object!",
      "DirectSound Error");
    CleanupDirectSound();
    return FALSE;
  }

  // Set the cooperation level for the DS object
  if (m_pDirectSound->SetCooperativeLevel(GetSafeHwnd(),
    DSSCL_NORMAL) != DS_OK) {
    MessageBox("Could not set cooperative level!",
      "DirectSound Error");
    CleanupDirectSound();
    return FALSE;
  }

  // Initialize the DS buffers
  if (!InitDSBuffers()) {
    MessageBox("Could not initialize DirectSound buffers!",
      "DirectSound Error");
    CleanupDirectSound();
    return FALSE;
  }

  return TRUE;
}

void CMainFrame::CleanupDirectSound() {
  // Cleanup the DS object
  if (m_pDirectSound) {
    m_pDirectSound->Release();
    m_pDirectSound = NULL;
  }
}

BOOL CMainFrame::InitDSBuffers() {
  // Initialize waves
  CWave waves[NUMSOUNDS];
  waves[0].Create(IDW_HELICOPTER);
  waves[1].Create(IDW_RICOCHET1);
  waves[2].Create(IDW_RICOCHET2);
  waves[3].Create(IDW_GUNFIRE1);
  waves[4].Create(IDW_GUNFIRE2);
  waves[5].Create(IDW_GUNFIRE3);
  waves[6].Create(IDW_GUNFIRE4);
  waves[7].Create(IDW_GUNFIRE5);
  waves[8].Create(IDW_EXPLODE1);
  waves[9].Create(IDW_EXPLODE2);
  waves[10].Create(IDW_EXPLODE3);
  waves[11].Create(IDW_EXPLODE4);
  waves[12].Create(IDW_SIREN);

  // Initialize DS buffers
  for (int i = 0; i < NUMSOUNDS; i++) {
    // Get the wave information
    DWORD         dwDataLen = waves[i].GetDataLen();
    WAVEFORMATEX  wfFormat;
    waves[i].GetFormat(wfFormat);

    // Setup the DS buffer description
    DSBUFFERDESC  dsbdDesc;
    ZeroMemory(&dsbdDesc, sizeof(DSBUFFERDESC));
    dsbdDesc.dwSize = sizeof(DSBUFFERDESC);
    dsbdDesc.dwFlags = DSBCAPS_CTRLDEFAULT | DSBCAPS_STATIC;
    dsbdDesc.dwBufferBytes = dwDataLen;
    dsbdDesc.lpwfxFormat = &wfFormat;

    // Create the DS buffer
    if (m_pDirectSound->CreateSoundBuffer(&dsbdDesc,
      &m_pDSBuffer[i], NULL) != DS_OK)
      return FALSE;

    // Lock the DS buffer
    BYTE* pDSBuffData;
    if (m_pDSBuffer[i]->Lock(0, dwDataLen, (LPLPVOID)&pDSBuffData,
      &dwDataLen, NULL, 0, 0) != DS_OK)
      return FALSE;

    // Write wave data to the DS buffer
    dwDataLen = waves[i].GetData(pDSBuffData, dwDataLen);

    // Unlock the DS buffer
    if (m_pDSBuffer[i]->Unlock(pDSBuffData, dwDataLen, NULL, 0) !=
      DS_OK)
      return FALSE;
  }

  return TRUE;
}

BOOL CMainFrame::RestoreDSBuffers() {
  // Restore the buffers
  for (int i = 0; i < NUMSOUNDS; i++)
    if (m_pDSBuffer[i]->Restore() != DS_OK)
      return FALSE;

  // Re-initialize the buffers
  return InitDSBuffers();
}

//-----------------------------------------------------------------
// CMainFrame Message Handler Methods
//-----------------------------------------------------------------
int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) {
  // Initialize DirectSound
  if (!InitDirectSound())
    return -1;

  // Seed the random number generator
  ::srand((UINT)timeGetTime());

  // Set the timer
  m_uiTimerID = SetTimer(IDT_SOUND, 500, NULL);
  if (m_uiTimerID == 0)
    return -1;

  return 0;
}

void CMainFrame::OnDestroy() {
  // Kill the timer
  KillTimer(m_uiTimerID);
}

void CMainFrame::OnPaint() {
  // Get the DC
  CPaintDC dc(this);

  // Calculate where to draw the text
  CString sText("Close your eyes, crank the volume, and hunker down for war...");
  CRect   rcClient;
  CSize   sizText;
  CPoint  ptTextPos;
  GetClientRect(&rcClient);
  sizText = dc.GetTextExtent(sText, sText.GetLength());
  ptTextPos.x = (rcClient.Width() - sizText.cx) / 2;
  ptTextPos.y = (rcClient.Height() - sizText.cy) / 2;

  // Draw the text
  dc.TextOut(ptTextPos.x, ptTextPos.y, sText);
}

void CMainFrame::OnTimer(UINT nIDEvent) {
  if (nIDEvent == m_uiTimerID) {
    // See if we should play a sound (50-50 chance)
    if (::rand() % 2) {
      // Determine which sound to play
      int nIndex = ::rand() % NUMSOUNDS;
      if (m_pDSBuffer[nIndex] != NULL) {
        DWORD dwStatus;
        m_pDSBuffer[nIndex]->GetStatus(&dwStatus);

        // Check to make sure the buffer hasn't been lost
        if (dwStatus & DSBSTATUS_BUFFERLOST)
          RestoreDSBuffers();

        // Check to make sure the sound isn't already playing
        if (!(dwStatus & DSBSTATUS_PLAYING)) {
          if (nIndex > 0) {
            // Set the panning of the sound
            m_pDSBuffer[nIndex]->SetPan((::rand() % 2000) - 1000);

            // Set the volume of the sound
            m_pDSBuffer[nIndex]->SetVolume((::rand() % 3) * -250);

            // Play the sound
            m_pDSBuffer[nIndex]->Play(0, 0, 0);
          }
          else {
            // Set the panning of the helicopter
            LONG lStart = (::rand() % 2) ? -1 : 1;
            m_lHeliPan = lStart * 8000;
            m_lHeliPanInc = -(lStart * 400);
            m_pDSBuffer[nIndex]->SetPan(m_lHeliPan);

            // Play the helicopter sound
            m_pDSBuffer[nIndex]->Play(0, 0, DSBPLAY_LOOPING);
          }
        }
      }
    }

    // If helicopter sound is playing, see if we should stop it
    DWORD dwStatus;
    m_pDSBuffer[0]->GetStatus(&dwStatus);
    if (dwStatus & DSBSTATUS_LOOPING) {
      if ((m_lHeliPan < -8000) || (m_lHeliPan > 8000))
        m_pDSBuffer[0]->Stop();
      else {
        m_lHeliPan += m_lHeliPanInc;
        m_pDSBuffer[0]->SetPan(m_lHeliPan);
        m_pDSBuffer[0]->SetVolume(-(::abs(m_lHeliPan) / 5));
      }
    }
  }
}
